# sample_four.py

import sys
import os
import platform
import wx
import wx.adv
from wx.lib.mixins.inspection   import InspectionMixin

__version__ = "0.1"

# def _displayHook
# class MyPanel
# class MyFrame
# class MyApp

#---------------------------------------------------------------------------

# Install a custom displayhook to keep Python from setting the global
# _ (underscore) to the value of the last evaluated expression.
# If we don't do this, our mapping of _ to gettext can get overwritten.
# This is useful/needed in interactive debugging with PyShell.
def _displayHook(obj):
    """
    Custom display hook to prevent Python stealing '_'.
    """

    if obj is not None:
        print(repr(obj))

# Add translation macro to builtin similar to what gettext does.
import builtins
builtins.__dict__['_'] = wx.GetTranslation

#---------------------------------------------------------------------------

class MyPanel(wx.Panel):
    """
    ...
    """
    def __init__(self, parent):
        super(MyPanel, self).__init__(parent)

        #------------
        
        # Simplified init method.
        self.SetProperties()
        self.CreateCtrls()
        self.BindEvents()
        self.DoLayout()
        
    #-----------------------------------------------------------------------

    def SetProperties(self):
        """
        Set the main panel properties (color...).
        """

        color = self.GetTopLevelParent().GetBackgroundColour()     
        self.SetBackgroundColour(color)

        
    def CreateCtrls(self):
        """
        ...
        """
        
        bitmap_dir = wx.GetApp().GetBitmapsDir()
        img_path = os.path.join(bitmap_dir, "wxWidgets.png")
        bitmap = wx.Bitmap(img_path, type=wx.BITMAP_TYPE_PNG)
        self.bitmap = wx.StaticBitmap(self, bitmap=bitmap)

        self.label = wx.StaticText(self, label=_("Hello"))
        
        self.langlbl = wx.StaticText(self, label=_("Language :"))
        font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        font.SetWeight(wx.BOLD)
        self.langlbl.SetFont(font)

        self.label1 = wx.StaticText(self, label=_("A nice label for test."))

        self.searchC = wx.SearchCtrl(self, id=wx.ID_ANY)

        #------------

        langList = ["English (U.K.)", "French"]
        langList.sort()
        
        self.langCh = wx.ComboBox(self,
                                  choices=langList,
                                  style=wx.CB_READONLY,
                                  size=(240, -1))
        font = wx.Font(9, wx.SWISS, wx.NORMAL, wx.NORMAL)
        self.langCh.SetFont(font)
        self.langCh.SetForegroundColour(wx.BLUE)
        self.langCh.Layout()       

        config = wx.GetApp().GetConfig()        
        lang_name = config.Read("Language")
        
        if lang_name == "LANGUAGE_ENGLISH":
            print("\n       Program language : %s ---> %s" % (lang_name, "en") )
            self.langCh.SetValue("English (U.K.)")

        elif lang_name == "LANGUAGE_FRENCH":
            print("\n       Program language : %s ---> %s" % (lang_name, "fr") )
            self.langCh.SetValue("French")            
            
        #------------
            
        font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        font.SetWeight(wx.BOLD)
        font.SetPointSize(9)
        
        self.btnOpen = wx.Button(self, id=wx.ID_OPEN,
                                 label=_("&Open a file"),
                                 size=(-1, -1))
        self.btnOpen.SetForegroundColour(wx.RED)
        self.btnOpen.SetFont(font)
     
        self.btnClose = wx.Button(self, id=wx.ID_CLOSE,
                                  label=_("&Close"),
                                  size=(-1, -1))
        self.btnClose.SetForegroundColour(wx.BLACK)
        self.btnClose.SetFont(font)
        self.btnClose.SetFocus()
        

    def BindEvents(self):
        """
        Bind all the events related to my panel.
        """

        self.Bind(wx.EVT_COMBOBOX, self.OnLang, self.langCh)
        self.Bind(wx.EVT_BUTTON, self.OnOpenDlg, self.btnOpen)
        self.Bind(wx.EVT_BUTTON, self.OnClose, self.btnClose)
        
        
    def DoLayout(self):
        """
        ...
        """
        
        vsizer = wx.BoxSizer(wx.VERTICAL)
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        langsz = wx.BoxSizer(wx.HORIZONTAL)
        
        #------------
        
        hsizer.AddStretchSpacer()
        hsizer.Add(self.label, 0, wx.ALIGN_CENTER)
        hsizer.AddStretchSpacer()

        #------------

        langsz.AddStretchSpacer()
        langsz.Add(self.langlbl, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
        langsz.Add(self.langCh, 0, wx.ALL, 5)
        langsz.AddStretchSpacer()

        #------------
        
        vsizer.AddStretchSpacer()
        vsizer.Add(self.bitmap, 0, wx.ALIGN_CENTER|wx.ALL, 5)
        vsizer.Add(self.label1, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        vsizer.Add(self.searchC, 0, wx.ALIGN_CENTER|wx.ALL, 10)
        vsizer.Add(hsizer, 0, wx.EXPAND)
        vsizer.Add(langsz, 0, wx.EXPAND|wx.ALL, 5)
        vsizer.Add(self.btnOpen, 1, wx.LEFT|wx.RIGHT|wx.EXPAND, 5)
        vsizer.Add(self.btnClose, 1, wx.LEFT|wx.RIGHT|wx.EXPAND, 5)
        vsizer.AddStretchSpacer()

        #------------
        
        self.SetSizer(vsizer)
        self.Layout()

        
    def OnLang(self, event):
        """
        ...
        """
        
        langue = self.langCh.GetValue()

        if langue == "English (U.K.)":
            val = "LANGUAGE_ENGLISH"
            print("\n       Program language : %s ---> %s" % (val, "en"))

        elif langue == "French":
            val = "LANGUAGE_FRENCH"
            print("\n       Program language : %s ---> %s" % (val, "fr"))

        config = wx.GetApp().GetConfig() 
        config.Write("Language", val)


    def OnOpenDlg(self, event):
        """
        ...
        """
        
        wildcard = "Python source (*.py)|*.py|"     \
                   "Compiled Python (*.pyc)|*.pyc|" \
                   "SPAM files (*.spam)|*.spam|"    \
                   "Egg file (*.egg)|*.egg|"        \
                   "All files (*.*)|*.*"
        
        with wx.FileDialog(
            self, message=_("Select a file"),
            defaultDir=os.getcwd(), 
            defaultFile="",
            wildcard=wildcard,
            style=wx.FD_OPEN | wx.FD_MULTIPLE |
                  wx.FD_CHANGE_DIR | wx.FD_FILE_MUST_EXIST |
                  wx.FD_PREVIEW
            ) as dlg:
            # Show the dialog and retrieve the user response.  
            # If it is the OK response, process the data.
            if dlg.ShowModal() == wx.ID_OK:
                # This returns a Python list of files that 
                # were selected.
                paths = dlg.GetPaths()


    def OnClose(self, event):
        """
        ...
        """
        
        self.GetParent().Close()
        event.Skip()
        
#---------------------------------------------------------------------------
                
class MyFrame(wx.Frame):
    """
    Main application window.
    """
    def __init__(self, parent):
        super(MyFrame, self).__init__(parent,
                                      style=wx.DEFAULT_FRAME_STYLE ^
                                           (wx.RESIZE_BORDER))

        #------------
        
        # Simplified init method.
        self.SetProperties()
        self.CreateMenu()
        self.CreateCtrls()
        self.BindEvents()
        self.DoLayout()
       
    #-----------------------------------------------------------------------      

    def SetProperties(self):
        """
        Set the main frame properties (title, icon...).
        """
     
        self.SetTitle(_("I18N - Sample App"))
        self.SetBackgroundColour("")
        
        icons_dir = wx.GetApp().GetIconsDir()
        icon_path = os.path.join(icons_dir, "wxwin.ico")
        frameIcon = wx.Icon(icon_path, type=wx.BITMAP_TYPE_ICO)

        self.SetIcon(frameIcon)


    def CreateMenu(self):
        """
        ...
        """
        
        menubar = wx.MenuBar()

        #------------
        
        # File menu.
        fileMenu = wx.Menu()
        closeMenuItem = fileMenu.Append(-1,
                                        _("Close"),
                                        _("Close the application"))
        self.Bind(wx.EVT_MENU, self.OnClose, closeMenuItem)
        menubar.Append(fileMenu, _("&File"))

        #------------
        
        # Edit menu.
        manageMenu = wx.Menu()
        manageSomethingMenuItem = manageMenu.Append(-1,
                                            _("Edit something"),
                                            _("Edit an entry of something"))
        self.Bind(wx.EVT_MENU, self.DoEditSomething, manageSomethingMenuItem)
        
        menubar.Append(manageMenu, _("&Edit"))

        #------------
        
        # Help menu.
        helpMenu = wx.Menu()
        aboutMenuItem = helpMenu.Append(-1,
                                        _("&About"),
                                        _("About the program"))
        self.Bind(wx.EVT_MENU, self.DoAboutBox, aboutMenuItem)
        menubar.Append(helpMenu, _("&Help"))
        
        self.SetMenuBar(menubar)


    def CreateCtrls(self):
        """
        ...
        """
        
        # Add a panel so it looks the correct on all platforms.
        self.panel = MyPanel(self)

        
    def BindEvents(self):
        """
        Bind all the events related to my frame.
        """

        self.Bind(wx.EVT_SIZE, self.OnResize)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        
    def DoLayout(self):
        """
        ...
        """
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.panel, 1, wx.EXPAND)
        self.SetSizer(sizer)
        self.SetInitialSize((300, 380))

                       
    def OnResize(self, event):        
        """
        ...
        """
        
        self.Refresh() 
        event.Skip()


    def OnClose(self, event):
        """
        ...
        """
        
        self.Close()
        event.Skip()

        
    def DoEditSomething(self, event):
        """
        ...
        """
        
        dlg = wx.MessageDialog(self,
                               _('Hello from Python and wxPython !'),
                               _('Edit'),
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

        
    def DoAboutBox(self, event):
        """
        ...
        """
        
        dlg = wx.MessageDialog(self,
                               _('Hello from Python and wxPython !'),
                               _('About'),
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()
        
        
    def OnCloseWindow(self, event):
        """
        ...
        """
        
        self.Destroy()
        event.Skip()

#---------------------------------------------------------------------------

class MyApp(wx.App, InspectionMixin):
    """
    The main application class.
    """
    def OnInit(self):
        """
        Default wx.App initialization.
        """

        # Work around for Python stealing "_".
        sys.displayhook = _displayHook

        self.installDir = os.path.split(os.path.abspath(sys.argv[0]))[0]

        # Used to identify app in $HOME/
        self.SetAppName("I18NTestApp")

        # Get Language from last run if set.
        config = wx.GetApp().GetConfig()
        language = config.Read("Language", "LANGUAGE_DEFAULT")
        self.language = language        

        # Simplified init method.
        self.Init()  # InspectionMixin (Ctrl+Alt+I).
        self.InitI18n()
        self.Setlang(language)
        
        # Local is not setup so we can create things 
        # that may need it to retrieve translations.
        self.frame = MyFrame(None)
        self.SetTopWindow(self.frame)
        self.frame.Show(True)

        return True

    #-----------------------------------------------------------------------

    def InitI18n(self):
        """
        ...
        """
        
        # Set locale for wxWidgets.
        # You would define wx.Locale in your wx.App.OnInit class.
        # Setup the Locale
        self.locale = wx.Locale(getattr(wx, self.language))
        path = os.path.abspath("./locale") + os.path.sep
        self.locale.AddCatalogLookupPathPrefix(path)
        self.locale.AddCatalog(self.GetAppName())

        print('\n\nAPPLICATION :\n')
        print(" Name ------> %s" % self.GetAppName())

        print('\n\nLOCALE :\n')
        print(" Canonical Name ------>", self.locale.GetCanonicalName())  # fr_FR
        print(" Language ID ------>", self.locale.GetLanguage())  # 78
        print(" Locale ------>", self.locale.GetLocale())  # French
        print(" Name ------>", self.locale.GetName())  # fr_FR
        

    def GetVersion(self):
        """
        Return the current MyApp version.
        """

        return __version__

    
    def GetInstallDir(self):
        """
        Returns the installation directory for MyApp.
        """
        
        return self.installDir        
        

    def GetIconsDir(self):
        """
        Returns the icons directory for MyApp.
        """
        
        icons_dir = os.path.join(self.installDir, "icons")        
        return icons_dir 


    def GetBitmapsDir(self):
        """
        Returns the bitmaps directory for MyApp.
        """
        
        bitmaps_dir = os.path.join(self.installDir, "bitmaps")        
        return bitmaps_dir

    
    def GetConfigDir(self):
        """
        Returns the option directory for MyApp.
        """
        
        config_dir  = os.path.join(self.installDir, "config")                  
        return config_dir

    
    def GetConfig(self):
        """
        Returns the configuration for MyApp.
        """
        
        if not os.path.exists(self.GetConfigDir()):
            # Create the config folder, it still doesn't exist.
            os.makedirs(self.GetConfigDir())
            
        config_file = wx.FileConfig(localFilename=os.path.join(
                                    self.GetConfigDir(), "options"))

        # On first run we default to English.
        if not config_file.HasEntry("Language"):
            config_file.Write(key="Language", value="LANGUAGE_DEFAULT")
        config_file.Flush()
        
        return config_file
    
   
    def Setlang(self, language):
        """
        To get some language settings to display properly on Linux.
        Thanks to Ianare Sevi.
        """

        if language == "LANGUAGE_ENGLISH":
            if platform.system() == "Linux":
                try:
                    # To get some language settings to display properly.
                    os.environ["LANGUAGE"] = "en"
                    
                except (ValueError, KeyError):
                    pass

        elif language == "LANGUAGE_FRENCH":
            if platform.system() == "Linux":
                try:
                    # To get some language settings to display properly.
                    os.environ["LANGUAGE"] = "fr"
        
                except (ValueError, KeyError):
                    pass
   
#---------------------------------------------------------------------------
                
def main():
    app = MyApp(False)
    app.MainLoop()
    
#---------------------------------------------------------------------------
        
if __name__ == "__main__":
    main()
