# geni18n.py

"""

Author  : Werner F. Bruhin <wernerfbd@gmx.ch>
          Modified and updated by Ecco
          
This will generate the .pot and .mo files for the application domain and
languages defined below.

The .po and .mo files are placed as per convention in

"appfolder/locale/lang/LC_MESSAGES"

The .pot file is placed in the locale folder.

This script or something similar should be added to your build process.

The actual translation work is normally done using a tool like poEdit or
similar, it allows you to generate a particular language catalog from the .pot
file or to use the .pot to merge new translations into an existing language
catalog.

"""

import os
import sys
import subprocess
import wx

# Language domain.
langDomain = "i18nwxapp"
# Languages you want to support.
supLang = {"en": wx.LANGUAGE_ENGLISH,
           "de": wx.LANGUAGE_GERMAN,
           "fr": wx.LANGUAGE_FRENCH,
          }

#---------------------------------------------------------------------------

# We remove English as source code strings are in English.
supportedLang = []
for l in supLang:
    if l != "en":
        supportedLang.append(l)

appFolder = os.getcwd()

#---------------------------------------------------------------------------

# Setup some stuff to get at Python I18N tools/utilities.
pyExe = sys.executable
pyFolder = os.path.split(pyExe)[0]
pyToolsFolder = os.path.join(pyFolder, "Tools")
pyI18nFolder = os.path.join(pyToolsFolder, "i18n")
pyGettext = os.path.join(pyI18nFolder, "pygettext.py")
pyMsgfmt = os.path.join(pyI18nFolder, "msgfmt.py")

# Create folder (locale).
if not os.path.exists("locale"):
    os.mkdir("locale")

outFolder = os.path.join(appFolder, "locale")

#---------------------------------------------------------------------------

# Build command for pygettext.
gtOptions = "-a -d %s -o %s.pot -p %s %s"
tCmd = pyExe + " " + pyGettext + " " + (gtOptions % (langDomain,
                                                     langDomain,
                                                     outFolder,
                                                     appFolder))

print ("Generating the .pot file")
print ("cmd: %s" % tCmd)
rCode = subprocess.call(tCmd)
print ("return code: %s\n\n" % rCode)

#---------------

for tLang in supportedLang:
    # Create folders (lang\LC_MESSAGES).
    if not os.path.exists("locale\%s" % tLang):
        os.mkdir("locale\%s" % tLang)
    if not os.path.exists("locale\%s\LC_MESSAGES" % tLang):
        os.mkdir("locale\%s\LC_MESSAGES" % tLang)
        
    # Build command for msgfmt.
    langDir = os.path.join(appFolder, ("locale\%s\LC_MESSAGES" % tLang))
    poFile = os.path.join(langDir, langDomain + ".po")
    tCmd = pyExe + " " + pyMsgfmt + " " + poFile
    
    print ("Generating the .mo file")
    print ("cmd: %s" % tCmd)
    rCode = subprocess.call(tCmd)
    print ("return code: %s\n\n" % rCode)
